@@ -17,16 +17,13 @@ import redis
 
 class Cache(object):
 
-    def __init__(self, expire_time):
-        self.expire_time = expire_time
-
     def exists(self, name):
         pass
 
     def get(self, name):
         pass
 
-    def set(self, name, value):
+    def set(self, name, value, expire_time=None):
         pass
 
     def delete(self, name):
@@ -37,20 +34,22 @@ class MemCache(Cache):
 
     __dictionary = dict()
 
-    def __init__(self, expire_time):
-        super(MemCache, self).__init__(expire_time)
-
     def exists(self, name):
         return self.__dictionary.has_key(name)
 
     def get(self, name):
         return self.__dictionary.get(name)
 
-    def set(self, name, value):
+    def set(self, name, value, expire_time=None):
         self.__dictionary[name] = value
 
     def delete(self, name):
-        self.__dictionary.pop(name)
+        try:
+            self.__dictionary.pop(name)
+        except:
+            import traceback
+            traceback.print_exc()
+            pass
 
 
 class RedisCache(Cache):
@@ -64,8 +63,8 @@ class RedisCache(Cache):
         'max_connections': 50,
     }
 
-    def __init__(self, expire_time=300):
-        super(RedisCache, self).__init__(expire_time)
+    def __init__(self):
+        super(RedisCache, self).__init__()
         self.redis_cli = redis.Redis(connection_pool=self.get_pool())
 
     def exists(self, name):
@@ -80,13 +79,14 @@ class RedisCache(Cache):
         else:
             return self.redis_cli.get(name)
 
-    def set(self, name, value):
+    def set(self, name, value, expire_time=None):
         pipe = self.redis_cli.pipeline()
         if isinstance(value, dict):
             pipe.hmset(name, value)
         else:
             pipe.set(name, str(value))
-        pipe.expire(name, self.expire_time)
+        if expire_time is not None:
+            pipe.expire(name, expire_time)
         pipe.execute()
 
     def delete(self, name):
@@ -164,30 +164,61 @@ class CacheName(object):
         return "_".join(arg_values)
 
     @classmethod
-    def get_by_function(cls, name, func, *func_args, **func_kwargs):
+    def get_by_function(cls, func, *func_args, **func_kwargs):
+        """根据func的函数名和参数来生成缓存key名
+        :param func: 函数对象
+        :param func_args: 函数位置参数
+        :param func_kwargs: 函数关键字参数
+        :return: string -- 缓存key名
+        """
         ns, func_type = cls.get_name_prefix(func, *func_args)
         args_key = cls.get_func_args_key(func_type, *func_args, **func_kwargs)
-        if name is None:
-            return "%s:%s" % (ns, args_key)
+        if args_key:
+            return ns, "%s:%s" % (ns, args_key)
         else:
-            return "%s:%s:%s" % (ns, args_key, name)
+            return ns, ns
+
+    @classmethod
+    def get_by_function_and_ns(cls, ns, func, *func_args, **func_kwargs):
+        """根据指定的ns和func的参数来生成缓存key名
+        :param ns: 指定的名字空间（前缀）
+        :param func: 函数对象
+        :param func_args: 函数位置参数
+        :param func_kwargs: 函数关键字参数
+        :return: string -- 缓存key名
+        """
+        args_key = cls.get_func_args_key(cls.get_func_type(func), *func_args, **func_kwargs)
+        return "%s:%s" % (ns, args_key) if args_key else ns
 
 
-def cacheable(name=None, expire=300, cache_cls=RedisCache):
+CACHE_GET = 1
+CACHE_DELETE = 2
+
+
+def cacheable(name=None, expire=300, action=CACHE_GET, cache_cls=RedisCache):
     def _cacheable(func):
         @functools.wraps(func)
         def wrapper(*args, **kwargs):
-            cache = cache_cls(expire_time=expire)
-            cache_key = CacheName.get_by_function(name, func, *args, **kwargs)
-            cache_value = cache.get(cache_key)
-            if cache_value is not None:
-                return cPickle.loads(cache_value)
+            cache = cache_cls()
 
-            result = func(*args, **kwargs)
-            if result is None:
+            if action == CACHE_GET:
+                ns, cache_key = CacheName.get_by_function(func, *args, **kwargs)
+                cache_value = cache.get(cache_key)
+                if cache_value is not None:
+                    return cPickle.loads(cache_value)
+                result = func(*args, **kwargs)
+                if result is None:
+                    return result
+                cache.set(cache_key, cPickle.dumps(result), expire)
+                if name is not None:
+                    cache.set("cache:alias:%s" % name, ns)  # 存储缓存ns的别名
+                return result
+            elif action == CACHE_DELETE:
+                result = func(*args, **kwargs)
+                if name is not None:
+                    ns = cache.get("cache:alias:%s" % name)
+                    cache_key = CacheName.get_by_function_and_ns(ns, func, *args, **kwargs)
+                    cache.delete(cache_key)
                 return result
-
-            cache.set(cache_key, cPickle.dumps(result))
-            return result
         return wrapper
     return _cacheable
